# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2017-03-31 17:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('botmanager', '0007_task_is_persistent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='is_persistent',
            field=models.BooleanField(default=True, verbose_name='\u0412\u044b\u043f\u043e\u043b\u043d\u044f\u0442\u044c \u043f\u043e\u043a\u0430 \u043d\u0435 \u0437\u0430\u0432\u0435\u0440\u0448\u0438\u0442\u0441\u044f \u0443\u0441\u043f\u0435\u0448\u043d\u043e'),
        ),
    ]
