# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2017-05-31 13:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('botmanager', '0009_task_extra_params'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='failed_action',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='Action, \u043d\u0430 \u043a\u043e\u0442\u043e\u0440\u043e\u043c \u043f\u0440\u043e\u0438\u0437\u043e\u0448\u043b\u0430 \u043e\u0448\u0438\u0431\u043a\u0430'),
        ),
    ]
