# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-12-08 15:29
from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('botmanager', '0002_auto_20161208_1406'),
    ]

    operations = [
        migrations.RenameField(
            model_name='task',
            old_name='retry_count',
            new_name='attempt_count',
        ),
        migrations.RemoveField(
            model_name='task',
            name='last_retry_dt',
        ),
        migrations.RemoveField(
            model_name='task',
            name='max_retry_count',
        ),
        migrations.RemoveField(
            model_name='task',
            name='retry_period',
        ),
        migrations.AddField(
            model_name='task',
            name='attempt_period',
            field=models.DurationField(default=datetime.timedelta(0, 3600), verbose_name='\u041f\u0435\u0440\u0438\u043e\u0434 \u043c\u0435\u0436\u0434\u0443 \u043f\u043e\u043f\u044b\u0442\u043a\u0430\u043c\u0438'),
        ),
        migrations.AddField(
            model_name='task',
            name='last_attempt_dt',
            field=models.DateTimeField(null=True, verbose_name='\u041f\u043e\u0441\u043b\u0435\u0434\u043d\u044f\u044f \u043f\u043e\u043f\u044b\u0442\u043a\u0430 \u0432\u044b\u043f\u043e\u043b\u043d\u0435\u043d\u0438\u044f'),
        ),
        migrations.AddField(
            model_name='task',
            name='max_attempt_count',
            field=models.IntegerField(null=True, verbose_name='\u041c\u0430\u043a\u0441\u0438\u043c\u0430\u043b\u044c\u043d\u043e\u0435 \u043a\u043e\u043b-\u0432\u043e \u043f\u043e\u043f\u044b\u0442\u043e\u043a \u0432\u044b\u043f\u043e\u043b\u043d\u0435\u043d\u0438\u0435 (\u0435\u0441\u043b\u0438 None, \u0442\u043e \u0431\u0435\u0441\u043a\u043e\u043d\u0435\u0447\u043d\u043e)'),
        ),
    ]
