# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-12-19 19:31
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('botmanager', '0003_auto_20161208_1529'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='create_dt',
            field=models.DateTimeField(auto_now_add=True, verbose_name='\u0414\u0430\u0442\u0430 \u0441\u043e\u0437\u0434\u0430\u043d\u0438\u044f \u0437\u0430\u0434\u0430\u0447\u0438'),
        ),
        migrations.AlterField(
            model_name='task',
            name='failed_action',
            field=models.CharField(blank=True, max_length=32, null=True, verbose_name='Action, \u043d\u0430 \u043a\u043e\u0442\u043e\u0440\u043e\u043c \u043f\u0440\u043e\u0438\u0437\u043e\u0448\u043b\u0430 \u043e\u0448\u0438\u0431\u043a\u0430'),
        ),
        migrations.AlterField(
            model_name='task',
            name='finish_dt',
            field=models.DateTimeField(blank=True, null=True, verbose_name='\u0414\u0430\u0442\u0430 \u0437\u0430\u0432\u0435\u0440\u0448\u0435\u043d\u0438\u044f'),
        ),
        migrations.AlterField(
            model_name='task',
            name='input',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='task',
            name='last_attempt_dt',
            field=models.DateTimeField(blank=True, null=True, verbose_name='\u041f\u043e\u0441\u043b\u0435\u0434\u043d\u044f\u044f \u043f\u043e\u043f\u044b\u0442\u043a\u0430 \u0432\u044b\u043f\u043e\u043b\u043d\u0435\u043d\u0438\u044f'),
        ),
        migrations.AlterField(
            model_name='task',
            name='last_error',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='task',
            name='last_error_dt',
            field=models.DateTimeField(blank=True, null=True, verbose_name='\u0414\u0430\u0442\u0430 \u043f\u043e\u0441\u043b\u0435\u0434\u043d\u0435\u0433\u043e fail-\u0430'),
        ),
        migrations.AlterField(
            model_name='task',
            name='max_attempt_count',
            field=models.IntegerField(blank=True, null=True, verbose_name='\u041c\u0430\u043a\u0441\u0438\u043c\u0430\u043b\u044c\u043d\u043e\u0435 \u043a\u043e\u043b-\u0432\u043e \u043f\u043e\u043f\u044b\u0442\u043e\u043a \u0432\u044b\u043f\u043e\u043b\u043d\u0435\u043d\u0438\u0435 (\u0435\u0441\u043b\u0438 None, \u0442\u043e \u0431\u0435\u0441\u043a\u043e\u043d\u0435\u0447\u043d\u043e)'),
        ),
        migrations.AlterField(
            model_name='task',
            name='output',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
    ]
