# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2017-04-25 16:51
from __future__ import unicode_literals

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('botmanager', '0008_auto_20170331_1752'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='extra_params',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
    ]
