# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-12-08 14:06
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('botmanager', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='last_error',
            field=jsonfield.fields.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='task',
            name='last_error_dt',
            field=models.DateTimeField(null=True, verbose_name='\u0414\u0430\u0442\u0430 \u043f\u043e\u0441\u043b\u0435\u0434\u043d\u0435\u0433\u043e fail-\u0430'),
        ),
    ]
